ggQQ <- function(LM) # argument: a linear model
{
    y <- quantile(LM$resid[!is.na(LM$resid)], c(0.25, 0.75))
    x <- qnorm(c(0.25, 0.75))
    slope <- diff(y)/diff(x)
    int <- y[1L] - slope * x[1L]
    p <- ggplot(LM, aes(sample=.resid)) +
        stat_qq(alpha = 0.5) +
        geom_abline(slope = slope, intercept = int, color="blue")

    return(p)
}
shinyServer(function(input, output,session) {
            
    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: some R libraries not found, will use base graphics</h4>"))
        NULL       
    })
    
   data <- reactive({
        x <- seq(0,10,length=100)
        if(input$which=="Linear Model") {
            for(i in input$step) {
                y <- 10 + x + rnorm(100) + input$step*x^2/25
                return(data.frame(x=x,y=y))
            }
        }        
        if(input$which=="Normal Residuals") {
            for(i in input$step) {
                y <- 10 + x + 2*rt(100,11-input$step)
                return(data.frame(x=x,y=y))
            }
        }        
        if(input$which=="Equal Variance") {
            for(i in input$step) {
                y <- 10 + rnorm(100,x,1+x*input$step)  + rnorm(100) 
                return(data.frame(x=x,y=y))
            }
        }        
            
   })     
    
    
   output$plot1 <- renderPlot({
        x <- data()$x
        y <- data()$y
        fit <- lm(y~x)
        if(input$whichgraph=="base" | !gotLibs()[3]) {
            par(mfrow=c(1,2))
            plot(x,y,pch=20)
            abline(fit,lwd=3)
            if(input$which=="Normal Residuals") {
                qqnorm(residuals(fit,pch="",main=""))
                qqline(residuals(fit))
            }    
            else {
              plot(fitted(fit),residuals(fit),xlab="Fitted Values",ylab="Residuals",pch=20)
              abline(h=0,lwd=3)
            }
        }
        else {
            dta <- data.frame(x=x,y=y)
            plt1 <- ggplot(dta,aes(x,y))+geom_point(size=2)+
                    geom_smooth(method="lm",se=FALSE,size=1.2)
            if(input$which=="Normal Residuals") 
               plt2 <- ggQQ(fit)
            else {
              dta <- data.frame(x=fitted(fit),y=residuals(fit))
              plt2 <- ggplot(dta,aes(x,y))+geom_point(size=2)+
                      xlab("Fitted Values")+ylab("Residuals")+
                      geom_abline(intercept=0,slope=0,size=1.2)
            }    
            pushViewport(viewport(layout = grid.layout(1, 2)))
            print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
            print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))
        
        }       
   })
           
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#assumptionsLSR\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/28/2014</h6>")
      
  })
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the assumptions of least squares regression
        <br>for each assumption it runs a movie, starting with the case where the assumption is fullfilled
        <br>and making things worse and worse</h5>"
  })

})

